﻿using System;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata;
using Microsoft.Extensions.Configuration;

#nullable disable

namespace WebApplication3.Models
{
    public partial class AquiferContext : DbContext
    {
        public AquiferContext()
        {
        }

        public AquiferContext(DbContextOptions<AquiferContext> options)
            : base(options)
        {
        }

        public virtual DbSet<Acuifero> Acuiferos { get; set; }
        public virtual DbSet<Pozo> Pozos { get; set; }
        public virtual DbSet<Recarga> Recargas { get; set; }
        public virtual DbSet<Servicio> Servicios { get; set; }
        public virtual DbSet<Simulacion> Simulaciones { get; set; }
        public virtual DbSet<TsPozo> TsPozos { get; set; }
        public virtual DbSet<TsRecarga> TsRecargas { get; set; }
        public virtual DbSet<BalanceHidrico> BalancesHidricos { get; set; }
        public virtual DbSet<DatosBalanceIn> DatosBalanceIns { get; set; }
        public virtual DbSet<DatosBalanceOut> DatosBalanceOuts { get; set; }
        public virtual DbSet<ConexionZonasIn> ConexionZonasIns { get; set; }
        public virtual DbSet<ConexionZonasOut> ConexionZonasOuts { get; set; }
        public virtual DbSet<Piezometria> Piezometria { get; set; }
        public virtual DbSet<Piezometer> Piezometers { get; set; }
        public virtual DbSet<TsPiezometer> TsPiezometers { get; set; }
        public virtual DbSet<ServiceQueue> ServiceQueue { get; set; }


        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {
                IConfigurationRoot configuration = new ConfigurationBuilder()
                                    .SetBasePath(AppDomain.CurrentDomain.BaseDirectory)
                                    .AddJsonFile("appsettings.json")
                                    .Build();

                optionsBuilder.UseMySql(configuration.GetConnectionString("AquiferDatabase"), Microsoft.EntityFrameworkCore.ServerVersion.Parse("5.7.24-mysql"));
            }
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.HasCharSet("latin1")
                .UseCollation("latin1_spanish_ci");

            modelBuilder.Entity<Acuifero>(entity =>
            {
                entity.ToTable("acuifero");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.Fichero)
                    .HasMaxLength(255)
                    .HasColumnName("fichero");

                entity.Property(e => e.Nombre)
                    .IsRequired()
                    .HasMaxLength(50)
                    .HasColumnName("nombre")
                    .HasDefaultValueSql("''");

                entity.Property(e => e.WorldOriginX).HasColumnName("world_origin_x");
                entity.Property(e => e.WorldOriginY).HasColumnName("world_origin_y");
                entity.Property(e => e.Angle).HasColumnName("angle");
                entity.Property(e => e.Length).HasColumnName("length");
                entity.Property(e => e.Height).HasColumnName("height");
                entity.Property(e => e.RechargeGeometry).HasColumnName("recharge_geometry");
                entity.Property(e => e.RechargeColorMap).HasColumnName("recharge_color_map");




                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<Pozo>(entity =>
            {
                entity.ToTable("pozo");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdAquifero)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_aquifero");

                entity.Property(e => e.IdSimulacion)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_simulacion");

                entity.Property(e => e.Latitud).HasColumnName("latitud");

                entity.Property(e => e.Longitud).HasColumnName("longitud");

                entity.Property(e => e.Maximo).HasColumnName("maximo");

                entity.Property(e => e.Minimo).HasColumnName("minimo");

                entity.Property(e => e.Nombre)
                    .HasMaxLength(50)
                    .HasColumnName("nombre");

                entity.Property(e => e.X)
                    .HasMaxLength(50)
                    .HasColumnName("x");   

                entity.Property(e => e.Y)
                    .HasMaxLength(50)
                    .HasColumnName("y");

                entity.Property(e => e.Z)
                    .HasMaxLength(50)
                    .HasColumnName("z");

                entity.Property(e => e.Prof)
                    .HasMaxLength(50)
                    .HasColumnName("prof");

                entity.Property(e => e.ScreenTop)
                    .HasMaxLength(50)
                    .HasColumnName("screen_top");

                entity.Property(e => e.ScreenBot)
                    .HasMaxLength(50)
                    .HasColumnName("screen_bot");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<Recarga>(entity =>
            {
                entity.ToTable("recarga");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdSimulacion)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_simulacion");

                entity.Property(e => e.IdAcuifero)
                .HasColumnType("int(10) unsigned")
                .HasColumnName("id_acuifero");

                entity.Property(e => e.Nombre)
                    .HasMaxLength(50)
                    .HasColumnName("nombre");

                entity.Property(e => e.Latitud).HasColumnName("latitud");

                entity.Property(e => e.Longitud).HasColumnName("longitud");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<Servicio>(entity =>
            {
                entity.ToTable("servicio");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.Pid)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("pid");

                entity.Property(e => e.Terminado)
                    .HasColumnType("tinyint(1) unsigned")
                    .HasColumnName("terminado");

                entity.Property(e => e.Inicio).HasColumnName("inicio");

                entity.Property(e => e.Fin).HasColumnName("fin");

                entity.Property(e => e.Tipo).HasColumnName("tipo");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<ServiceQueue>(entity =>
            {
                entity.ToTable("service_queue");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdSimulacion)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_simulacion");

                entity.Property(e => e.IdAcuifero)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_acuifero");

                entity.Property(e => e.Terminado)
                    .HasColumnType("tinyint(1) unsigned")
                    .HasColumnName("terminado");

                entity.Property(e => e.Error)
                    .HasColumnType("tinyint(1) unsigned")
                    .HasColumnName("error");

                entity.Property(e => e.Corriendo)
                    .HasColumnType("tinyint(1) unsigned")
                    .HasColumnName("corriendo");

                entity.Property(e => e.Inicio).HasColumnName("inicio");

                entity.Property(e => e.Fin).HasColumnName("fin");


                entity.HasKey(e => e.Id)
                .HasName("id");
            });


            modelBuilder.Entity<Simulacion>(entity =>
            {
                entity.ToTable("simulacion");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdAcuifero)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_acuifero");

                entity.Property(e => e.Nombre)
                    .HasMaxLength(255)
                    .HasColumnName("nombre");
                
                entity.Property(e => e.IsolineImage)
                    .HasMaxLength(255)
                    .HasColumnName("isoline_image");

                entity.Property(e => e.Geojson)
                    .HasMaxLength(255)
                    .HasColumnName("geojson");

                entity.Property(e => e.SimulacionAcuifero)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("simulacion_acuifero")
                    .HasDefaultValueSql("0");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<TsPozo>(entity =>
            {
                entity.ToTable("ts_pozo");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdPozo)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_pozo");

                entity.Property(e => e.MarcaTiempo)
                    .HasMaxLength(50)
                    .HasColumnName("marca_tiempo");

                entity.Property(e => e.MarcaTiempoEnd)
                    .HasMaxLength(50)
                    .HasColumnName("marca_tiempo_end");

                entity.Property(e => e.InfoComplementaria)
                    .HasMaxLength(255)
                    .HasColumnName("info_complementaria");

                entity.Property(e => e.Valor).HasColumnName("valor");

                
                entity.Property(e => e.DateIn).HasColumnName("date_in");
                entity.Property(e => e.DateEnd).HasColumnName("date_end");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<TsRecarga>(entity =>
            {
                entity.ToTable("ts_recarga");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdRecarga)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_recarga");

                entity.Property(e => e.MarcaTiempo)
                    .HasMaxLength(50)
                    .HasColumnName("marca_tiempo");

                entity.Property(e => e.MarcaTiempoEnd)
                    .HasMaxLength(50)
                    .HasColumnName("marca_tiempo_end");

                entity.Property(e => e.InfoComplementaria)
                    .HasMaxLength(255)
                    .HasColumnName("info_complementaria");

                entity.Property(e => e.Valor).HasColumnName("valor");


                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<BalanceHidrico>(entity =>
            {
                entity.ToTable("balance_hidrico");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdSimulacion)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_simulacion");

                entity.Property(e => e.StressPeriod).HasColumnName("stress_period");

                entity.Property(e => e.Zona).HasColumnName("zona");

                entity.Property(e => e.TotalZona).HasColumnName("total_zona");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<DatosBalanceIn>(entity =>
            {
                entity.ToTable("datos_balance_in");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdBalance)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_balance_hidrico");

                entity.Property(e => e.Storage).HasColumnName("storage");

                entity.Property(e => e.Recharge).HasColumnName("recharge");

                entity.Property(e => e.Drains).HasColumnName("drains");
                entity.Property(e => e.Wells).HasColumnName("wells");


                entity.Property(e => e.ConstantHead).HasColumnName("constant_head");


                entity.Property(e => e.Total).HasColumnName("total");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<DatosBalanceOut>(entity =>
            {
                entity.ToTable("datos_balance_out");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdBalance)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_balance_hidrico");

                entity.Property(e => e.Storage).HasColumnName("storage");

                entity.Property(e => e.Recharge).HasColumnName("recharge");

                entity.Property(e => e.Drains).HasColumnName("drains");
                entity.Property(e => e.Wells).HasColumnName("wells");

                entity.Property(e => e.ConstantHead).HasColumnName("constant_head");


                entity.Property(e => e.Total).HasColumnName("total");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<ConexionZonasIn>(entity =>
            {
                entity.ToTable("conexion_zonas_in");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdDatosBalanceIn)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_balance_in");

                entity.Property(e => e.From).HasColumnName("from");

                entity.Property(e => e.To).HasColumnName("to");

                entity.Property(e => e.Value).HasColumnName("value");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<ConexionZonasOut>(entity =>
            {
                entity.ToTable("conexion_zonas_out");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdDatosBalanceOut)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_balance_out");

                entity.Property(e => e.From).HasColumnName("from");

                entity.Property(e => e.To).HasColumnName("to");

                entity.Property(e => e.Value).HasColumnName("value");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<Piezometria>(entity =>
            {
                entity.ToTable("piezometria");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.IdSimulacion)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id_simulacion");

                entity.Property(e => e.StressPeriod).HasColumnName("stress_period");

                entity.Property(e => e.Layer).HasColumnName("layer");

                entity.Property(e => e.Columns).HasColumnName("columns");
                
                entity.Property(e => e.Rows).HasColumnName("rows");

                entity.Property(e => e.Original).HasColumnName("original");
                
                entity.Property(e => e.Convertida).HasColumnName("convertida");

                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<Piezometer>(entity =>
            {
                entity.ToTable("piezometer");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.Name)
                    .HasMaxLength(50)
                    .HasColumnName("nombre");

                entity.Property(e => e.X)
                    .HasMaxLength(50)
                    .HasColumnName("x");

                entity.Property(e => e.Y)
                    .HasMaxLength(50)
                    .HasColumnName("y");

                entity.Property(e => e.Z)
                    .HasMaxLength(50)
                    .HasColumnName("z");

                entity.Property(e => e.Prof)
                    .HasMaxLength(50)
                    .HasColumnName("prof");

                entity.Property(e => e.ScreenTop)
                    .HasMaxLength(50)
                    .HasColumnName("screen_top");

                entity.Property(e => e.ScreenBot)
                    .HasMaxLength(50)
                    .HasColumnName("screen_bot");


                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            modelBuilder.Entity<TsPiezometer>(entity =>
            {
                entity.ToTable("ts_piezometer");

                entity.Property(e => e.Id)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("id");

                entity.Property(e => e.PiezometerId)
                    .HasColumnType("int(10) unsigned")
                    .HasColumnName("piezometer_id");

                entity.Property(e => e.Date)
                    .HasMaxLength(50)
                    .HasColumnName("date");

                entity.Property(e => e.Day)
                    .HasMaxLength(50)
                    .HasColumnName("day");

                entity.Property(e => e.Quote)
                    .HasMaxLength(50)
                    .HasColumnName("quote");


                entity.HasKey(e => e.Id)
                .HasName("id");
            });

            OnModelCreatingPartial(modelBuilder);
        }

        partial void OnModelCreatingPartial(ModelBuilder modelBuilder);
    }
}
